from import_export.resources import ModelResource
from import_export.fields import Field
from import_export.widgets import Widget

from .models import TimelineLog


class JSONWidget(Widget):

    def render(self, value, obj=None):
        return value


class TimelineLogResource(ModelResource):
    extra_data = Field(
        attribute='extra_data',
        column_name='extra_data',
        widget=JSONWidget(),
        default=None
    )

    class Meta:
        model = TimelineLog



from __future__ import unicode_literals

from appconf import AppConf
from django.conf import settings  # noqa
from django.utils.translation import ugettext_lazy as _


class TimelineLoggerConf(AppConf):

    DEFAULT_TEMPLATE = 'timeline_logger/default.txt'

    PAGINATE_BY = 25

    USER_EMAIL_FIELD = 'email'

    DIGEST_EMAIL_RECIPIENTS = None
    DIGEST_EMAIL_SUBJECT = _('Events timeline')
    DIGEST_FROM_EMAIL = None

    class Meta:
        prefix = 'timeline'

    def configure_digest_email_recipients(self, value):
        if value is None:
            return []
        return value

    def configure_digest_from_email(self, value):
        if value is None:
            return settings.DEFAULT_FROM_EMAIL
        return value



from django.template import Library

register = Library()


@register.simple_tag
def render_message(log_entry, template=None, **context):
    return log_entry.get_message(template=template, **context)



from django.contrib import admin

from .models import TimelineLog


@admin.register(TimelineLog)
class TimelineLogAdmin(admin.ModelAdmin):
    list_display = ['__str__', 'timestamp', 'template']
    list_filter = ['timestamp', 'content_type']
    list_select_related = ['content_type']



import sys

# Safely importing the HTML parser utility, highly dependent on Python versions
if sys.version_info >= (3, 4):
    # From v3.4.0 in advance
    import html
else:
    # Python 2.x versions
    from HTMLParser import HTMLParser
    html = HTMLParser()



import logging
from datetime import timedelta

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.mail import send_mail
from django.core.management.base import BaseCommand, CommandError
from django.template.loader import render_to_string
from django.utils import timezone
from django.utils.html import strip_tags
from django.utils.translation import ugettext_lazy as _

from timeline_logger.compat import html
from timeline_logger.models import TimelineLog

logger = logging.getLogger('timeline_logger')


class Command(BaseCommand):
    help = 'Sends mail notifications for last events to (admin) users.'
    template_name = 'timeline_logger/notifications.html'

    def add_arguments(self, parser):
        parser.add_argument(
            '--days', type=int,
            help='An integer number with the number of days to look at from today to the past.'
        )

        recipients_group = parser.add_mutually_exclusive_group()
        recipients_group.add_argument('--all', action='store_true', help=_('Send e-mail to all users'))
        recipients_group.add_argument('--staff', action='store_true', help=_('Send e-mail to staff users'))
        recipients_group.add_argument(
            '--recipients-from-setting', action='store_true',
            help=_('Send e-mail to adresses listed in settings.TIMELINE_DIGEST_EMAIL_RECIPIENTS')
        )

    def handle(self, *args, **options):
        recipients = self.get_recipients(**options)
        queryset = self.get_queryset(**options)

        if not queryset.exists():
            logger.info('No logs in timeline. No emails sent.')
            return

        self.send_email(recipients, queryset)

    def get_queryset(self, **options):
        """
        Filters the list of log objects to display
        """
        days = options.get('days')
        queryset = TimelineLog.objects.order_by('-timestamp')
        if days:
            try:
                start = timezone.now() - timedelta(days=days)
            except TypeError:
                raise CommandError("Incorrect 'days' parameter. 'days' must be a number of days.")
            else:
                return queryset.filter(timestamp__gte=start)

        return queryset

    def get_recipients(self, **options):
        """
        Figures out the recipients
        """
        if options['recipients_from_setting']:
            return settings.TIMELINE_DIGEST_EMAIL_RECIPIENTS

        users = get_user_model()._default_manager.all()
        if options['staff']:
            users = users.filter(is_staff=True)
        elif not options['all']:
            users = users.filter(is_staff=True, is_superuser=True)
        return users.values_list(settings.TIMELINE_USER_EMAIL_FIELD, flat=True)

    def get_context(self, queryset):
        return {
            'logs': queryset,
            'start_date': queryset[0].timestamp,
        }

    def send_email(self, recipients, queryset):
        context = self.get_context(queryset)
        html_content = render_to_string(self.template_name, context)
        text_content = html.unescape(strip_tags(html_content))

        send_mail(
            subject=settings.TIMELINE_DIGEST_EMAIL_SUBJECT,
            message=text_content,
            from_email=settings.TIMELINE_DIGEST_FROM_EMAIL,
            recipient_list=recipients,
            fail_silently=False,
            html_message=html_content
        )
        logger.info('Notification emails sent to: %s', recipients)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.9 on 2018-03-13 17:38
from __future__ import unicode_literals

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('timeline_logger', '0003_auto_20171025_1414'),
    ]

    operations = [
        migrations.AlterField(
            model_name='timelinelog',
            name='content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='contenttypes.ContentType', verbose_name='content type'),
        ),
        migrations.AlterField(
            model_name='timelinelog',
            name='extra_data',
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True, verbose_name='extra data'),
        ),
        migrations.AlterField(
            model_name='timelinelog',
            name='object_id',
            field=models.TextField(blank=True, null=True, verbose_name='object id'),
        ),
        migrations.AlterField(
            model_name='timelinelog',
            name='timestamp',
            field=models.DateTimeField(auto_now_add=True, verbose_name='timestamp'),
        ),
        migrations.AlterField(
            model_name='timelinelog',
            name='user',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='user'),
        ),
    ]



import logging

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.contrib.postgres.fields import JSONField
from django.db import models
from django.template.loader import get_template, render_to_string
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext, ugettext_lazy as _

from .conf import settings

logger = logging.getLogger('timeline_logger')


DEFAULT_TEMPLATE = settings.TIMELINE_DEFAULT_TEMPLATE


@python_2_unicode_compatible
class TimelineLog(models.Model):
    content_type = models.ForeignKey(
        ContentType,
        verbose_name=_('content type'),
        on_delete=models.SET_NULL,
        blank=True, null=True,
    )
    object_id = models.TextField(
        verbose_name=_('object id'),
        blank=True, null=True
    )
    content_object = GenericForeignKey('content_type', 'object_id')
    timestamp = models.DateTimeField(verbose_name=_('timestamp'), auto_now_add=True)
    extra_data = JSONField(
        verbose_name=_('extra data'),
        null=True, blank=True,
    )
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        verbose_name=_('user'),
        on_delete=models.SET_NULL,
        blank=True, null=True,
    )
    template = models.CharField(max_length=200, default=DEFAULT_TEMPLATE)

    class Meta:
        verbose_name = _('timeline log entry')
        verbose_name_plural = _('timeline log entries')

    def __str__(self):
        if self.object_id:
            return "{ct} - {pk}".format(
                ct=self.content_type.name,
                pk=self.object_id
            )

        return ugettext('TimelineLog Object')

    @classmethod
    def log_from_request(cls, request, content_object, template=None, **extra_data):
        """
        Given an ``HTTPRequest`` object and a generic content, it creates a
        ``TimelineLog`` object to store the data of that request.

        :param request: A Django ``HTTPRequest`` object.
        :param content_object: A Django model instance. Any object can be
        related.
        :param template: String representing the template to be used to render
        the message. Defaults to 'default.txt'.
        :param extra_data: A dictionary (translatable into JSON) determining
        specifications of the action performed.
        :return: A newly created ``TimelineLog`` instance.
        """
        try:
            user = request.user
        except AttributeError:
            user = None

        if template is not None:
            # Ensure that the expected template actually exists.
            get_template(template)

        timeline_log = cls.objects.create(
            content_object=content_object,
            extra_data=extra_data or None,
            template=template or DEFAULT_TEMPLATE,
            user=user,
        )
        logger.debug('Logged event in %s %s', content_object._meta.object_name, content_object.pk)
        return timeline_log

    def get_message(self, template=None, **extra_context):
        """
        Gets the 'log' message, describing the event performed.

        :param template: String representing the template to be used to render
        the message. Defaults to ``self.template``.
        :return: The log message string.
        """
        context = {'log': self}
        context.update(**extra_context)
        return render_to_string(template or self.template, context)



from __future__ import absolute_import, unicode_literals

from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class TimelineLoggerConfig(AppConfig):
    name = 'timeline_logger'
    verbose_name = _('Django Timeline Logger')

    def ready(self):
        from . import conf  # noqa



#!/usr/bin/env python

# To use a consistent encoding
from codecs import open
from os import path

from setuptools import find_packages, setup

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='django-timeline-logger',
    version='1.1',
    description='Generic event logger for Django models.',
    long_description=long_description,
    author='Maykin Media',
    author_email='support@maykinmedia.nl',
    url='https://github.com/maykinmedia/django-timeline-logger',
    install_requires=[
        'Django>=1.11',
        'django-appconf',
    ],
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    zip_safe=False,
    setup_requires=['pytest-runner'],
    tests_require=[
        'factory-boy',
        'psycopg2',
        'pytest',
        'pytest-cov',
        'pytest-django',
        'pytest-pep8',
        'pytest-pylint',
        'pytest-pythonpath',
        'pytest-runner',
    ],
    extras_require={
        'docs': [
            'sphinx',
            'sphinx_rtd_theme'
        ],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Application Frameworks'
    ]
)



from django.conf import settings
from django.views.generic import ListView

from .models import TimelineLog


class TimelineLogListView(ListView):
    queryset = TimelineLog.objects.order_by('-timestamp')
    paginate_by = settings.TIMELINE_PAGINATE_BY



# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-06-24 10:45
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('timeline_logger', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='timelinelog',
            options={'verbose_name': 'timeline log entry', 'verbose_name_plural': 'timeline log entries'},
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-06-15 18:16
from __future__ import unicode_literals

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='TimelineLog',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('timestamp', models.DateTimeField(auto_now_add=True)),
                ('extra_data', django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True)),
                ('template', models.CharField(default='timeline_logger/default.txt', max_length=200)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]
